# Networking Providers
#
# Copyright (C) 2014 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Radek Novacek <rnovacek@redhat.com>
#
"""
System power state management.

Usage:
    %(cmd)s list
    %(cmd)s suspend
    %(cmd)s hibernate
    %(cmd)s reboot [--force]
    %(cmd)s poweroff [--force]

Commands:
    list             Prints a list of available power states.
    suspend          Suspend the system (suspend to RAM).
    hibernate        Hibernate the system (suspend to disk).
    reboot           Shutdown and reboot the system (--force will skip shutdown of running services).
    poweroff         Shutdown the system (--force will skip shutdown of running services).

Options:
    --force          Skip shutting down services first
"""

import abc

from lmi.scripts.common import command
from lmi.scripts.common import errors
from lmi.scripts.common.formatter import command as fcmd
from lmi.scripts.powermanagement import *

def cmd_list(ns):
    """
    Implementation of 'power list' command.
    """
    state_names = {
        POWER_STATE_POWEROFF: "poweroff",
        POWER_STATE_POWEROFF_FORCE: "force poweroff",
        POWER_STATE_REBOOT: "reboot",
        POWER_STATE_REBOOT_FORCE: "force reboot",
        POWER_STATE_SUSPEND: "suspend",
        POWER_STATE_HIBERNATE: "hibernate"
    }

    available_states = list_power_states(ns)
    for state, name in state_names.items():
        if state in available_states:
            yield (name, "yes")
        else:
            yield (name, "no")

def _do_switch_power_state(ns, cmd):
    """
    Executes given command on particular host and logs it.
    """
    res = switch_power_state(ns, cmd)
    LOG().info('Machine %s has been %s.', ns.connection.uri,
            { POWER_STATE_POWEROFF       : "shut down"
            , POWER_STATE_POWEROFF_FORCE : "forcefully shut down"
            , POWER_STATE_REBOOT         : "rebooted"
            , POWER_STATE_REBOOT_FORCE   : "forcefully rebooted"
            , POWER_STATE_SUSPEND        : "suspended"
            , POWER_STATE_HIBERNATE      : "hibernated"}[cmd])
    return res

class Lister(command.LmiLister):
    CALLABLE = 'lmi.scripts.powermanagement.cmd:cmd_list'
    COLUMNS = ('PowerState','Available')

class PowerCmd(command.LmiCheckResult):
    EXPECT = 0
    OPT_NO_UNDERSCORES = True

    @abc.abstractmethod
    def get_cmd_code(self, force=False):
        pass
    
    def execute(self, ns, **kwargs):
        return _do_switch_power_state(ns, self.get_cmd_code(**kwargs))

class Suspend(PowerCmd):
    
    def get_cmd_code(self, force=False):
        return POWER_STATE_SUSPEND

class Hibernate(PowerCmd):

    def get_cmd_code(self, force=False):
        return POWER_STATE_HIBERNATE

class Poweroff(PowerCmd):

    def get_cmd_code(self, force=False):
        return POWER_STATE_POWEROFF_FORCE if force else POWER_STATE_POWEROFF

class Reboot(PowerCmd):

    def get_cmd_code(self, force=False):
        return POWER_STATE_REBOOT_FORCE if force else POWER_STATE_REBOOT

PowerManagement = command.register_subcommands(
    'PowerManagement', __doc__,
    {
        'list':       Lister,
        'suspend':    Suspend,
        'hibernate':  Hibernate,
        'poweroff':   Poweroff,
        'reboot':     Reboot
    },
)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from setuptools import setup

try:
    long_description = open('README.md', 'rt').read()
except IOError:
    long_description = ''

setup(
    name='openlmi-scripts-powermanagement',
    version='0.4.0',
    description='LMI command for power management.',
    long_description=long_description,
    author=u'Radek Novacek',
    author_email='rnovacek@redhat.com',
    url='https://github.com/openlmi/openlmi-scripts',
    download_url='https://github.com/openlmi/openlmi-scripts/tarball/master',
    platforms=['Any'],
    license="BSD",
    classifiers=[
        'License :: OSI Approved :: BSD License',
        'Operating System :: POSIX :: Linux',
        'Topic :: System :: Systems Administration',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Intended Audience :: Developers',
        'Environment :: Console',
    ],

    install_requires=['openlmi-tools >= 0.9.1'],

    namespace_packages=['lmi', 'lmi.scripts'],
    packages=['lmi', 'lmi.scripts', 'lmi.scripts.powermanagement'],
    include_package_data=True,

    entry_points={
        'lmi.scripts.cmd': [
                'power = lmi.scripts.powermanagement.cmd:PowerManagement',
            ],
        },
    )



# Copyright (c) 2014, Red Hat, Inc. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# The views and conclusions contained in the software and documentation are
# those of the authors and should not be interpreted as representing official
# policies, either expressed or implied, of the FreeBSD Project.

"""
LMI power management provider client library.
"""

from lmi.scripts.common.errors import LmiFailed, LmiInvalidOptions
from lmi.scripts.common import get_logger

import IPy

LOG = get_logger(__name__)

POWER_STATE_POWEROFF = 12
''' Turn off the system. '''

POWER_STATE_POWEROFF_FORCE = 8
''' Turn off the system without shutting down services first. '''

POWER_STATE_REBOOT= 15
''' Reboot the system. '''

POWER_STATE_REBOOT_FORCE = 5
''' Reboot the system without shutting down services first. '''

POWER_STATE_SUSPEND = 4
''' Suspend the system. '''

POWER_STATE_HIBERNATE = 7
''' Hibernate the system. '''


def list_power_states(ns):
    '''
    Get list of available power states.

    :return: list of power states
    :rtype: list of POWER_STATE_* constants
    '''
    capabilities = ns.LMI_PowerManagementCapabilities.first_instance()
    for state in capabilities.PowerStatesSupported:
        yield state

def switch_power_state(ns, state):
    '''
    Switch system power state.

    :param state: Requested power state.
    :type state: POWER_STATE_* constant
    '''
    # Check if the power state is available first
    capabilities = ns.LMI_PowerManagementCapabilities.first_instance()
    if not state in capabilities.PowerStatesSupported:
        raise LmiInvalidOptions("Power state is not supported.")
    service = ns.LMI_PowerManagementService.first_instance()
    service.RequestPowerStateChange(PowerState=state)
    return 0
